/*==============================================================================
   DEMO-HTS.C :- HTS Demo Program
   Sub Systems, Inc.
   Software License Agreement  (1995-2020)
   ----------------------------------
   This is a demo program.  The program shows how you can utilize HTS.DLL
   routine to perform html viewing/editing functions.
===============================================================================*/
#define  STRICT

#include "windows.h"
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if !defined(WIN32)
  #include "print.h"
  #if defined (__TURBOC__)
    typedef unsigned short USHORT;     // missing in compiler
  #endif
#endif

#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "string.h"

#include "commdlg.h"

#include "ter.h"
#include "ter_cmd.h"

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (WIN32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance
   #define FreeProcInstance(x)
   #define GET_WM_COMMAND_ID(wp,lp) LOWORD(wp)
   #define farmalloc  malloc
   #define farrealloc realloc
   #define farfree    free
#else
   #define GET_WM_COMMAND_ID(wp,lp) (wp)
#endif

#include "ter.h"
#include "hts.h"

#include "demo.h"


/****************************************************************************
   Main Windows Routine
****************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;
    int i;

    #ifndef WIN32
      int QueueLength=96;
      while (!SetMessageQueue(QueueLength) && (QueueLength-=8));
    #endif

    hPrevInst=hPrevInstance;
    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    SetMessageQueue(96);   // needed for OLE2

    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return (msg.wParam);
}

/****************************************************************************
   Initialize window data and register window class
****************************************************************************/

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    wc.style = 0;
    wc.lpfnWndProc = (void far *)DemoWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "DemoMenu";
    wc.lpszClassName = "DemoHtsClass";

    return (RegisterClass(&wc));
}


/****************************************************************************
    Save instance handle and create main window
****************************************************************************/

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    int i;
    HDC hDC;

    hInst = hInstance;

    hDemoWnd = CreateWindow(
        "DemoHtsClass",
        "HTS Demonstration",
        WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN,
        0,
        0,
        650,
        900,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (!hDemoWnd)
        return (FALSE);

    ShowWindow(hDemoWnd, nCmdShow);
    UpdateWindow(hDemoWnd);

    LoadTerControl();                // load the TER control into memory

    //****** initialize the structure variables to open an editor window *********
    for (i=0;i<MAX_WINDS;i++) {
        arg[i].x=20;                 // Initial X position of TER edit window
        arg[i].y=10;                 // Initial Y position of TER edit window
        arg[i].width=600;            // Initial edit window width
        arg[i].height=800;           // Initial edit window height
        arg[i].LineLimit=0;          // set to 0 for unlimited number of lines
        arg[i].InitLine=1;           // Initial line number to position on
        arg[i].WordWrap=TRUE;        // Word wrap on?
        arg[i].PrintView=FALSE;      // Wrap at window width
        arg[i].PageMode=FALSE;       // page mode off
        arg[i].FittedView=TRUE;      // Fitted view
        arg[i].InputType='F';        // Input type: (F)ile or (B)uffer
        arg[i].ShowStatus=FALSE;     // don't display the status
        arg[i].ShowMenu=FALSE;       // don't display menu
        arg[i].ShowVerBar=TRUE;      // display vertical scroll bar
        arg[i].ShowHorBar=FALSE;     // display horizontal scroll bar (N/A with word wrap)
        arg[i].UserCanClose=FALSE;   // user can close this window
        arg[i].ruler=FALSE;          // don't show the ruler
        arg[i].ToolBar=FALSE;        // don't show the tool bar
        arg[i].BorderMargins=TRUE;   // display margins around border
        arg[i].ReadOnly=TRUE;        // Do not allow edits
        lstrcpy(arg[i].file,"DEMO.HTM");// Html file name if the input type is 'F'
        arg[i].hBuffer=0;            // handle to the input/output buffer
        arg[i].SaveFormat=SAVE_DEFAULT; // format of the output file (keep it save as input)
        arg[i].hInst=hInst;          // Current application instant handle
        arg[i].hParentWnd=hDemoWnd;      // let this be the parent of the editor window
        arg[i].style=WS_CHILD;       //  Editor window style
        lstrcpy(arg[i].FontTypeFace,"Arial"); // default font type faces
        arg[i].PointSize=12;         //  point size of the default font
        arg[i].open=FALSE;           // all window closed in the beginning
    }


    // get the screen resolution
    hDC=GetDC(hDemoWnd);
    ResX=GetDeviceCaps(hDC,LOGPIXELSX);   // pixels per inch
    ResY=GetDeviceCaps(hDC,LOGPIXELSY);
    ReleaseDC(hDemoWnd,hDC);

    FocusWnd=-1;                           // no window created yet

    return (TRUE);
}

/****************************************************************************
    Process Main Window messages
****************************************************************************/

LRESULT CALLBACK _export DemoWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    DLGPROC lpProcParam;
    int  CommandId;
    BOOL result;
    int i,type;
    struct StrTerField field;

    switch (message) {

        case WM_INITMENU:                     // initialize menu
            if ((GetMenu(hWnd)!=(HMENU)wParam)) break;
            CurWnd=SelectWindow(hWnd,FALSE);
            DemoInitMenu((HMENU)wParam);
            break;

        case WM_COMMAND:
           // extract the command id from wParam
           CommandId=GET_WM_COMMAND_ID(wParam,lParam);

           // These menu options are directly supported by the HTS DLL
           if (CommandId>=HTM_FIRST_COMMAND && CommandId<=HTM_LAST_COMMAND) {
              HtsCommand(arg[CurWnd].hTextWnd,CommandId);
              break;
           }

           // The following menus are handled by the demo program
           switch (CommandId) {
              case IDM_OPEN:                 // open a new window
                 // find an unused window parameter block
                 for (CurWnd=0;CurWnd<MAX_WINDS;CurWnd++) if (!arg[CurWnd].open) break;
                 if (CurWnd==MAX_WINDS) {
                    MessageBox(hWnd,"All window parameter blocks in use!",NULL,MB_OK);
                    break;
                 }

                 lpProcParam = (DLGPROC)MakeProcInstance((FARPROC)DemoOpenDlg, hInst);
                 result=DialogBox(hInst,"DemoOpenDlg",hWnd,lpProcParam);
                 FreeProcInstance((FARPROC)lpProcParam);

                 if (result) OpenControl(hWnd); // Prepare parameters and call TER routine
                 break;


              case IDM_SAVE:                   // save the document
                 CurWnd=SelectWindow(hWnd,TRUE);

                 if (CurWnd>=0) {
                    if (arg[CurWnd].InputType=='B')
                         HtsSave(arg[CurWnd].hTextWnd,HTML_BUF,arg[CurWnd].file,&(arg[CurWnd].hBuffer),&(arg[CurWnd].BufferLen));
                    else HtsCommand(arg[CurWnd].hTextWnd,HTM_SAVE);
                 }
                 break;

              case IDM_CLOSE:                   // close a viewer window
                 CurWnd=SelectWindow(hWnd,TRUE);
                 if (CurWnd>=0) {
                    if (arg[CurWnd].InputType=='F') type=HTML_FILE;
                    else                            type=HTML_BUF;
                    HtsClose(arg[CurWnd].hTextWnd,type,arg[CurWnd].file,&(arg[CurWnd].hBuffer),&(arg[CurWnd].BufferLen));

                    if (CurWnd==FocusWnd) FocusWnd=-1;    // window with focus closed
                 }

                 break;

              case IDM_SWAP:
                 SwapPicture();break;          // swap the last picture in the document

              case IDM_SHOW_FRAMES:                  // test code here
                 CurWnd=SelectWindow(hWnd,TRUE);
                 TerSetFlags3(CurWnd>=0?arg[CurWnd].hTextWnd:NULL,TRUE,TFLAG3_SHOW_FRAMES);
                 break;

              case IDM_NO_SCRIPT:                   // disable scripts
                 HtsSetFlags(NULL,TRUE,HFLAG_NO_SCRIPT);
                 break;

              case IDM_NO_INTERNET:                  // disable internet file access
                 HtsSetFlags(NULL,TRUE,HFLAG_NO_INTERNET);
                 break;

              case IDM_DEBUG:             // toggle TE dll debug mode
                 PrevDebugMode=TerSetDebug(!PrevDebugMode);
                 break;

              case IDM_OTHER:                  // test code here
                 CurWnd=SelectWindow(hWnd,TRUE);
                 if (CurWnd>=0) DemoOther(arg[CurWnd].hTextWnd);
                 else           DemoOther(NULL);
                 break;

              default:
                return (DefWindowProc(hWnd, message, wParam, lParam));
           }
           break;

        case WM_SETFOCUS:
           for (i=0;i<MAX_WINDS;i++) if (arg[i].open && arg[i].hTextWnd==(HWND)wParam) break;
           if (i<MAX_WINDS) FocusWnd=i; // note the window that lost the focus
           break;

        case WM_CLOSE:                       // close all TER windows
            for (i=0;i<MAX_WINDS;i++) {
                if (arg[i].open) {
                   CurWnd=i;
                   if (arg[CurWnd].InputType=='F') type=HTML_FILE;
                   else                            type=HTML_BUF;
                   if (!HtsClose(arg[CurWnd].hTextWnd,type,arg[CurWnd].file,&(arg[CurWnd].hBuffer),&(arg[CurWnd].BufferLen))) break;
                }
            }
            if (i==MAX_WINDS) DestroyWindow(hWnd);  // all window successfully closed
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        case HTS_TITLE:   // set the title of the window, wParam=Window Handle, lParam=pointer to title string
            SetWindowText((HWND)wParam,(LPSTR)lParam);
            break;

        case HTS_LINK:    // load new file
            LoadNewFile((HWND)wParam,(LPBYTE)lParam);
            break;

        //case HTS_PICTURE: // 20061129: use the internal LocatePicture function; resolve picture path
        //    return (LRESULT) LocatePicture((HWND)wParam,(LPBYTE)lParam);

        case HTS_PICT_ID: // record picture id of the last picture
            for (CurWnd=0;CurWnd<MAX_WINDS;CurWnd++) if (arg[CurWnd].hTextWnd==(HWND)wParam) break;
            if (CurWnd<MAX_WINDS) {
               PictWinNo=CurWnd;
               hPictWnd=arg[CurWnd].hTextWnd;
               PictId=(int)lParam;
            }

            break;

        case HTS_FORM:    // handle form submission
            return (LRESULT) FormSubmission((HWND)wParam,(int)lParam);

        case TER_CLOSE:                      // TER sends this message before closing
            for (i=0;i<MAX_WINDS;i++) {      // mark this window as closed
                if (arg[i].open && arg[i].hTextWnd==(HWND)wParam) {
                    arg[i].open=FALSE;
                    break;
                }
            }
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (LRESULT)NULL;
}

/******************************************************************************
     Create the editor control and read the HTML data into it. Steps:
     1. Read the file into a buffer if requested.
     2. Create a control window of the TER_CLASS
     3. Initialize the HTS control data for this window
     4. Read the HTML data into this control.
*******************************************************************************/
OpenControl(HWND hWnd)
{
   DWORD style;
   HWND hTextWnd;
   BOOL result;
   LPBYTE pFileExt=NULL;
   int i,len;
   BOOL IsHtmlFile=FALSE;

   // note the file path
   lstrcpy(FilePath,arg[CurWnd].file);
   for (i=lstrlen(FilePath)-1;i>=0;i--) if (FilePath[i]=='\\' || FilePath[i]==':') break;
   FilePath[i+1]=0;     // the path name without the file name


   // Read the file into a buffer if requested
   if (arg[CurWnd].InputType=='B') {  /* In a real life application, you will use the
                                         buffer option when the data is located in
                                         a buffer form in a data base.  For the purpose
                                         of this demo, we will make a buffer from
                                         the file name you just supplied. */
      if (arg[CurWnd].hBuffer==0) {    // make a buffer if it does not already exist
         ReadFileIntoMemory(hWnd);
      }
      else {
         if (strcmp(arg[CurWnd].file,PrevFile)!=0) {
            GlobalFree(arg[CurWnd].hBuffer);
            arg[CurWnd].hBuffer=0;
            ReadFileIntoMemory(hWnd);
         }
      }
   }

   // Create a control window of TER_CLASS
   style=arg[CurWnd].style|WS_VISIBLE|TER_PAGE_MODE;
   if (style&WS_CHILD) style|=WS_CLIPSIBLINGS;

   if (arg[CurWnd].WordWrap)       style|=TER_WORD_WRAP;
   if (arg[CurWnd].FittedView)     style|=TER_FITTED_VIEW;
   if (arg[CurWnd].ReadOnly)       style|=TER_READ_ONLY;
   if (arg[CurWnd].ShowVerBar)     style|=TER_VSCROLL;
   if (arg[CurWnd].ShowHorBar)     style|=TER_HSCROLL;
   if (arg[CurWnd].BorderMargins)  style|=TER_BORDER_MARGIN;

   arg[CurWnd].hTextWnd = hTextWnd= CreateWindow(
          TER_CLASS,
          "Document",
          style,
          arg[CurWnd].x,
          arg[CurWnd].y,
          arg[CurWnd].width,
          arg[CurWnd].height,
          hWnd,
          NULL,
          hInst,
          NULL
          );

    if (arg[CurWnd].hTextWnd==NULL) {
       MessageBox(hWnd,"Error Creating the Editor Window!",NULL,MB_OK);
       return FALSE;
    }

    arg[CurWnd].open=TRUE;

   // Initialize the HTS data for this control
   if (!HtsInitialize(hTextWnd,hWnd)) {
       MessageBox(hWnd,"Error initializing the HTML data",NULL,MB_OK);
       return FALSE;
   }

   // Determine the file type by file extension
   len=lstrlen(arg[CurWnd].file);
   if (len>0) {
      if (len>=4) pFileExt=&(arg[CurWnd].file[len-4]);
      if (pFileExt && lstrcmpi(pFileExt,".HTM")==0) IsHtmlFile=TRUE;
      if (HtsIsHttpFile(arg[CurWnd].file)) IsHtmlFile=TRUE;

      //HtsSetFlags(arg[CurWnd].hTextWnd,false,HFLAG_WRITE_LINK_STYLE);   // an example of setting a flag
      if (IsHtmlFile) {  // Read html data into the control
         if (arg[CurWnd].InputType=='F') result=HtsRead(arg[CurWnd].hTextWnd,HTML_FILE,arg[CurWnd].file,NULL,0,NULL);
         else result=HtsRead(arg[CurWnd].hTextWnd,HTML_BUF,"",arg[CurWnd].hBuffer,arg[CurWnd].BufferLen,NULL);
      }
      else {             // possibly an RTF file, read it directly into TER dll
         if (arg[CurWnd].InputType=='F') result=ReadTerFile(arg[CurWnd].hTextWnd,arg[CurWnd].file);
         else result=SetTerBuffer(arg[CurWnd].hTextWnd,arg[CurWnd].hBuffer,arg[CurWnd].BufferLen,arg[CurWnd].file,TRUE);
      }

      //HtsWrite(arg[CurWnd].hTextWnd,HTML_FILE,"new.htm",null,null,false);   // an example of saving the file after reading
      if (!result) MessageBox(NULL,"File not found or file read error!",NULL,MB_OK);
   }

   FocusWnd=CurWnd;   // window with focus - used to rout menu commands to proper HTML window

   return TRUE;
}

/*****************************************************************************
     LoadNewFile:
     Load new file and position at the destination
*****************************************************************************/
LoadNewFile(HWND hWnd,LPBYTE path)
{
    BYTE dest[MAX_WIDTH+1]="";
    int len,i,MouseX,MouseY;
    BYTE string[MAX_WIDTH+1],MousePos[30]="";
    BOOL FullPath;
    BOOL IsInternetFile,IsMailTo;
    BYTE CurPath[MAX_WIDTH+1];

    IsInternetFile=(HtsIsHttpFile(path) || HtsIsHttpFile(FilePath));

    IsMailTo=(strstr(path,"mailto:")?TRUE:FALSE);

    if (IsMailTo) {
       MessageBox(hWnd,path,"Mail-to link",MB_OK);
       return TRUE;
    }

    if (!IsInternetFile) {
       // Extract the mouse position (stored as ?X,Y) from the path name
       len=lstrlen(path);
       for (i=len-1;i>=0;i--) if (path[i]=='?') break;

       if (i>=0) {
          path[i]=0;
          lstrcpy(MousePos,&(path[i+1]));

          // Extract mouse x and y position
          for (i=0;i<lstrlen(MousePos);i++) if (MousePos[i]==',') break;
          if (i<lstrlen(MousePos)) {     // ?X,Y string found
             MousePos[i]=0;
             MouseX=atoi(MousePos);    // not used in this example
             MouseY=atoi(&(MousePos[i+1]));
             wsprintf(string,"x: %d, y: %d",MouseX,MouseY);
             MessageBox(hWnd,string,"Pixel Position",MB_OK);
             if (lstrlen(path)==0) return FALSE;  // no link file specified
          }
       }

       // Extract destination name from the path name
       len=lstrlen(path);
       for (i=len-1;i>=0;i--) if (path[i]=='#') break;

       if (i>0) {
          path[i]=0;
          lstrcpy(dest,&(path[i+1]));
       }

       // check if full path specified
       FullPath=TRUE;
       len=lstrlen(path);
       for (i=0;i<len;i++) if (path[i]=='\\' || path[i]==':') break;
       if (i==len) FullPath=FALSE;


       // check if file exists
       if (FullPath) lstrcpy(string,path);  // full path in the path string
       else {                               // take the directory from the initial html file
          HtsGetCurPath(hWnd,CurPath);
          lstrcpy(string,CurPath);
          lstrcat(string,path);
       }
       lstrcpy(path,string);            // return the full path of the picture file
       if (access(string,6)==-1) {      // if a new file
          MessageBox(hWnd,"File Not Found!",path,MB_OK);
          return FALSE;
       }
    }

    // read the html file
    HtsClearWindow(hWnd);         // clear the existing text

    if (!HtsRead(hWnd,HTML_FILE,path,NULL,0,dest)) {
       MessageBox(NULL,"File not found or file read error!",NULL,MB_OK);
    }

    return TRUE;
}

/*****************************************************************************
     LocatePicture:
     This function is responsible for retriving a picture from the input source
     and returnig the path for a copy of the picture in the local machine. The
     local path string is copies to the 'path' argument.  Because, in the
     demo application the picture bitmap is alredy in the local machine, this
     function does not change the 'path' string.

     This function also returns the picture type using the PICT_ arguments.
*****************************************************************************/
LocatePicture(HWND hWnd,LPBYTE path)
{
    char string[MAX_WIDTH+1];
    int i,len,PictType=PICT_NONE;
    BOOL FullPath;

    if (HtsIsHttpFile(path) || HtsIsHttpFile(FilePath)) return PICT_UNKNOWN;  // let the dll determine the picture type

    lstrcpy(string,path);
    strupr(string);

    // demo determines picture type by file extension
    if      (strstr(string,".BMP")) PictType=PICT_BMP;
    else if (strstr(string,".GIF")) PictType=PICT_GIF;   // please note that a Unisys license is required to use the GIF and TIFF formats
    else if (strstr(string,".WMF")) PictType=PICT_WMF;
    else if (strstr(string,".TIF")) PictType=PICT_TIFF;
    else if (strstr(string,".JPG")) PictType=PICT_JPG;
    else if (strstr(string,".PNG")) PictType=PICT_PNG;

    // extact //localhost/ string if found
    if (strstr(string,"//LOCALHOST")==string) {
       lstrcpy(string,path);
       lstrcpy(path,&(string[lstrlen("//LOCALHOST/")]));
    }

    // check if full path specified
    FullPath=TRUE;
    len=lstrlen(path);
    for (i=0;i<len;i++) if (path[i]=='\\' || path[i]==':') break;
    if (i==len) FullPath=FALSE;


    // check if file exists
    if (FullPath) lstrcpy(string,path);  // full path in the path string
    else {                               // take the directory from the initial html file
       lstrcpy(string,FilePath);
       lstrcat(string,path);
    }
    lstrcpy(path,string);            // return the full path of the picture file
    if (access(string,4)==-1) {      // file not found
       //MessageBox(hWnd,path,"Picture File Not Found!",MB_OK);
       return PICT_NONE;
    }


    return PictType;
}

/*****************************************************************************
     FormSubmission:
     This function simply displays the currently submitted form and embedded
     control data.
*****************************************************************************/
FormSubmission(HWND hWnd,int FormNo)
{
    int  TotalFields,i,IntData,FieldType,len;
    char action[100],method[10],name[100];

    if ((TotalFields=HtsGetFormInfo(hWnd,FormNo,action,method))<0) return FALSE;

    wsprintf(FormData,"Form No: %d, Action: %s, Method: %s. ",FormNo,(LPSTR)action,(LPSTR)method);

    // get the name of the submit field
    if (HtsGetFieldData(hWnd,-1,-1,name,&IntData,TextData,sizeof(TextData)-1)<0) return FALSE;
    len=lstrlen(FormData);
    if (lstrlen(name)>0) wsprintf(&(FormData[len]),"Submit Field Name: %s. ",(LPSTR)name);

    // Extract each field
    for (i=0;i<TotalFields;i++) {
       // extract field type, field name, integer data, and text data
       if ((FieldType=HtsGetFieldData(hWnd,FormNo,i,name,&IntData,TextData,sizeof(TextData)-1))<0) continue;

       // append common field data to the display buffer
       len=lstrlen(FormData);
       wsprintf(&(FormData[len]),"Field# %d, name: %s, ",i,(LPBYTE)name);

       // process data for each field type
       if (FieldType==FTYPE_TEXT) {
          // append text data to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Text, data: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_RADIO) {
          // append text data to the display buffer
          lstrcat(FormData,"type: Radio, ");
          len=lstrlen(FormData);
          if (lstrlen(TextData)>0) wsprintf(&(FormData[len]),"val: %s, ",(LPBYTE)TextData);
          if (IntData) lstrcat(FormData,"Checked; ");
          else         lstrcat(FormData,"Unchecked; ");
       }
       else if (FieldType==FTYPE_SUBMIT) {
          // append field type to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Submit; value: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_RESET) {
          // append field type to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Reset; ");
       }
       else if (FieldType==FTYPE_TEXTAREA) {
          // append textarea data to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Textarea, data: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_CHECKBOX) {
          // append text data to the display buffer
          len=lstrlen(FormData);
          if (IntData) wsprintf(&(FormData[len]),"type: Checkbox, Checked; ");
          else         wsprintf(&(FormData[len]),"type: Checkbox, Unchecked; ");
       }
       else if (FieldType==FTYPE_HIDDEN) {
          // append hidden data to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Hidden, value: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_PASSWORD) {
          // append text data to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Password, data: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_IMAGE) {
          // append pixel location to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Image, x/y: %s; ",(LPBYTE)TextData);
       }
       else if (FieldType==FTYPE_SELECT) {
          // append the selected items to the display buffer
          len=lstrlen(FormData);
          wsprintf(&(FormData[len]),"type: Select, item(s): %s; ",(LPBYTE)TextData);
       }
    }



    MessageBox(NULL,FormData,"Submitted Form Data",MB_OK);

    return TRUE;
}

/*****************************************************************************
     Read the file into a buffer to be passed to the HTML reader.
*****************************************************************************/
ReadFileIntoMemory(HWND hWnd)
{
    char huge *buffer;
    long     FileSize;
    FILE *InStream;                    // to read the input file

    if (access(arg[CurWnd].file,6)==-1) {      // if a new file
       arg[CurWnd].hBuffer=GlobalAlloc(GMEM_MOVEABLE,1);
       buffer=GlobalLock(arg[CurWnd].hBuffer);
       buffer[0]=0xd;
       GlobalUnlock(arg[CurWnd].hBuffer);
       arg[CurWnd].BufferLen=1;
       return TRUE;
    }

    InStream=fopen(arg[CurWnd].file,"rb"); // open the file
    if (!InStream) {
        MessageBox(hWnd,"Error in opening file",NULL,MB_OK);return FALSE;}

    FileSize=filelength(fileno(InStream));
    if   (NULL==(arg[CurWnd].hBuffer=GlobalAlloc(GMEM_MOVEABLE,FileSize))
      || (NULL==(buffer=(char huge *)GlobalLock(arg[CurWnd].hBuffer)))) {
        MessageBox(hWnd,"Ran Out of Memory.",NULL,MB_OK);
        return FALSE;
    }

    arg[CurWnd].BufferLen=0;

    while (fread(DemoLine,1,1,InStream)==1) {
      /* read one byte at a time because 'fread' wouldn't work with
         a 'far' buffer in the small model */
      buffer[arg[CurWnd].BufferLen]=DemoLine[0];
      (arg[CurWnd].BufferLen)++;
    }

    fclose(InStream);

    if (arg[CurWnd].BufferLen==0) {buffer[0]=0xd;arg[CurWnd].BufferLen=1;}
    GlobalUnlock(arg[CurWnd].hBuffer);

    return TRUE;
}

/******************************************************************************
    DemoInitMenu:
    Enable/disable and check the menu items.
*******************************************************************************/
DemoInitMenu(HMENU hMenu)
{
    HWND hWnd=NULL;
    int i;

    if (CurWnd>=0) hWnd=arg[CurWnd].hTextWnd;  // current html window

    // set each menu item
    for (i=HTM_FIRST_COMMAND;i<=HTM_LAST_COMMAND;i++) {
       EnableMenuItem(hMenu,i,HtsMenuEnable(hWnd,i)? MF_ENABLED:MF_GRAYED);
       CheckMenuItem(hMenu,i,HtsMenuSelect(hWnd,i)? MF_CHECKED:MF_UNCHECKED);
    }

    // set location menu options
    EnableMenuItem(hMenu,IDM_SAVE,hWnd?MF_ENABLED:MF_GRAYED);
    EnableMenuItem(hMenu,IDM_CLOSE,hWnd?MF_ENABLED:MF_GRAYED);

    return TRUE;
}

/******************************************************************************
    OpenCurPrinter:
    Open the current printer.  This function returns the device context of
    the printer.
*******************************************************************************/
HDC OpenCurPrinter(HWND hWnd)
{
    char    PrinterName[60],    // name of the printer
            PrinterDriver[60],  // printer driver
            PrinterPort[60];    // printer port
    HDC hPrtDC;

    if (!CurrentPrinter(PrinterName,PrinterDriver,PrinterPort)) return NULL; // get current printer info

    // Open the printer device context
    PrinterDriver[lstrlen(PrinterDriver)-4]=0;            // strip the .drv extension
    if (NULL==(hPrtDC=CreateDC(PrinterDriver,PrinterName,PrinterPort,NULL))) {
        MessageBox(NULL,"Can not initialize printing","Device Context Error",MB_OK);
        return NULL;
    }

    return hPrtDC;
}

/******************************************************************************
    CurrentPrinter:
    Get information (name,driver,port) for the currently selected printer.
    The function returns FALSE if the current printer not set.
******************************************************************************/
BOOL CurrentPrinter(LPSTR name,LPSTR driver,LPSTR port)
{
    char PrintData[80],ext[5];
    LPSTR temp,name1,driver1=0,port1=0;
    int len;

    name[0]=driver[0]=port[0]=0;       // initialized the output variables

    if (!GetProfileString("windows","device",(LPSTR) "",PrintData,80)) return FALSE;

    temp=name1=PrintData;              // beginning pointer for the scan

    while(*temp) {                     // scan until a NULL is found
       if (*temp==',') {               // a delimited found
          *temp=0;                     // NULL terminate the previous string, and advance the pointer
          temp++;
          while (*temp==' ') temp=AnsiNext(temp);   // skip leading spaces
          if (!driver1) driver1=temp;  // Initiate the driver string if not found yet
          else {                       // else it is the last string, i.e. the port string
             port1=temp;
             break;                    // all strings parsed
          }
       }
       else temp=AnsiNext(temp);       // advance the pointer
    }

    lstrcpy(name,name1);               // copy the strings to the user specified area
    lstrcpy(port,port1);
    lstrcpy(driver,driver1);

    // add .drv extension to the driver name, if necessary
    len=lstrlen(driver);
    if (len<=4) lstrcat(driver,".drv");
    else {
       lstrcpy(ext,&driver[len-4]);
       if (lstrcmpi(ext,".DRV")!=0) lstrcat(driver,".DRV");
    }

    return TRUE;
}

/*****************************************************************************
     DemoOther:
     This function simply has some temporary test code
*****************************************************************************/
DemoOther(HWND hWnd)
{
   int i;

   //TerCommand(hWnd,ID_TABLE_SPLIT_CELL_VERT);

   HtsSetFlags3(hWnd,true,HFLAG3_WORD_SUPPORT)
;

   return TRUE;
}

/*****************************************************************************
     DemoPrint:
     Example of using the TerMergePrint function to print an HTML file.
*****************************************************************************/
DemoPrint(HWND hWnd)
{
     struct StrPrint prt;
     RECT rect;
     PRINT_CALLBACK proc;

     // set the print call back
     proc=(PRINT_CALLBACK) MakeProcInstance((FARPROC)PrintCallback, hInst);
     TerRegisterPrintCallback(proc);  // set the callback

     // set the StrPrint structure
     memset(&prt,0,sizeof(prt));

     prt.InputType='F';
     prt.file[0]=0;        // the file will be read by the PrintCallback function

     rect.left=rect.top=50;       // rectangle to print to in mm units
     rect.right=rect.bottom=200;
     prt.rect=&rect;

     prt.OnePage=TRUE;      // print only one page
     prt.hParentWnd=hWnd;   // set a parent window to receive all html messages

     TerMergePrint(&prt);

     return TRUE;
}

/*****************************************************************************
     PrintCallback:
     The TerMergePrint function calls this function. In this function
     we do the html initializing and read the html document.
*****************************************************************************/
int FAR PASCAL _export PrintCallback(HWND hParentWnd, HWND hTerWnd)
{
     HtsInitialize(hTerWnd,hParentWnd);
     HtsRead(hTerWnd,HTML_FILE,"demo.htm",NULL,0,NULL);  // read the demo document here

     return TRUE;
}

/*****************************************************************************
     SwapPicture:
     This function swaps the last picture in the open document with another
     picture.  This function illustrates the use of HtsSetPicture function.
*****************************************************************************/
SwapPicture()
{
    if (PictWinNo<0 || PictWinNo>=MAX_WINDS || arg[PictWinNo].hTextWnd!=hPictWnd) return FALSE;

    HtsSetPicture(arg[PictWinNo].hTextWnd,PictId,PICT_BMP,"demo1.bmp");

    return TRUE;
}

/******************************************************************************
                             Helper Routines
*******************************************************************************/
/******************************************************************************
    SelectWindow:
    Select an open window.
*******************************************************************************/
int SelectWindow(HWND hWnd, BOOL ShowDlg)
{
    int i,sel=-1,OpenCount=0;
    DLGPROC lpProcParam;
    HWND hChildWnd;

    // check if focus window is already set
    if (FocusWnd>=0 && FocusWnd<MAX_WINDS && arg[FocusWnd].open) return FocusWnd;

    // select the top level child window if avaialble
    hChildWnd=GetTopWindow(hWnd);
    for (i=0;i<MAX_WINDS;i++) if (arg[i].open && arg[i].hTextWnd==hChildWnd) return (FocusWnd=i);

    for (i=0;i<MAX_WINDS;i++) {    // select window to close
       if (arg[i].open) {
          OpenCount++;
          sel=i;
       }
    }
    if (OpenCount==0) return -1;

    if (OpenCount>1) {
       if (ShowDlg) {
          lpProcParam = (DLGPROC)MakeProcInstance((FARPROC)DemoSelectWin, hInst);
          sel=DialogBox(hInst,"DemoSelectWin",hWnd,lpProcParam);
          FreeProcInstance((FARPROC)lpProcParam);
       }
       else return -1;
    }

    return sel;
}

/******************************************************************************
    HugeMemMove:
    Use this function to move data from one buffer to another when the
    buffer may be larger than 64K.  For buffer smaller than 64K, the
    FarMove function will provide much better performance.
*******************************************************************************/
void HugeMemMove(char huge *src,char huge * dest, UINT count)
{
    UINT i;

    for (i=0;i<count;i++) dest[i]=src[i];
    return;
}

/******************************************************************************
     AssignIntValue:
     Extract an integer value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL AssignIntValue(HWND hWin,int ControlId,LPINT pInt)
{
    BOOL ErrorFlag;

    *pInt=GetDlgItemInt(hWin,ControlId,&ErrorFlag,(BOOL) 0);
    if (!ErrorFlag) {
       MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
       SetFocus(GetDlgItem(hWin,ControlId));
       return (FALSE);
    }
    return TRUE;
}

/******************************************************************************
     AssignLongValue:
     Extract a long value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL AssignLongValue(HWND hWin,int ControlId,LPLONG pLong)
{
    char string[20];
    int i;

    GetDlgItemText(hWin,ControlId,string,sizeof(string)-1);  // get the number

    for (i=0;i<strlen(string);i++) {  // check for non numeric characters
        if (string[i]=='-') continue;
        if (string[i]<'0' || string[i]>'9') {
           MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
           SetFocus(GetDlgItem(hWin,ControlId));
           return (FALSE);
        }
    }

    (*pLong)=atol(string);       // convert to long

    return TRUE;
}

/******************************************************************************
     SetDemoLong:
     Set a long value to a dialog field.
*******************************************************************************/
BOOL SetDemoLong(HWND hDlg,int ControlId,long value)
{
    char string[20];

    wsprintf(string,"%ld",value);
    SetDlgItemText(hDlg,ControlId,string);

    return TRUE;
}

/******************************************************************************
     GetDemoLong:
     Gety a long value from a dialog field.
*******************************************************************************/
BOOL GetDemoLong(HWND hDlg,int ControlId,long *value)
{
    GetDlgItemText(hDlg,ControlId,DemoLine,19);
    (*value)=atol(DemoLine);

    return TRUE;
}

/******************************************************************************
    OurPrintf:
    This routine formats and display a given set of arguments.  The format
    is given by the first argument.  The argument 2 to n contain the
    arguments for the specified format.  The function uses MessageBox to
    display the formatted string.
******************************************************************************/
int OurPrintf(LPBYTE fmt,...)
{
    LPSTR ArgStart;
    char string[MAX_WIDTH+2];
    HWND hWnd;

    ArgStart=(LPSTR) &fmt;  // pointer to first argument
    ArgStart=&ArgStart[4];  // go past the first argument
    wvsprintf(string,fmt,ArgStart);

    if (NULL!=(hWnd=FindWindow("DBWin",NULL))) { // debug window open
        if (NULL!=(hWnd=GetWindow(hWnd,GW_CHILD))) {    // get the edit window
           int len=lstrlen(string);
           string[len]=0xd;
           string[len+1]=0xa;
           string[len+2]=0;
           #if defined (WIN32)
              SendMessage(hWnd,EM_SETSEL,32767,32767);  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD_PTR)(LPSTR)string);
              SendMessage(hWnd,EM_SCROLLCARET,0,0);     // scroll caret into view
           #else
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD_PTR)(LPSTR)string);
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
           #endif
        }
    }
    else MessageBox(NULL,string,NULL,MB_OK);
    return TRUE;
}

/******************************************************************************
    Dialog box routines
******************************************************************************/

/****************************************************************************
    Process messages for "DemoOpenDlg" dialog box
    This dialog box accepts the parameters to be passed to TER routine.
    Finally this routines makes a call to TER routine to open a window.
****************************************************************************/
int CALLBACK _export DemoOpenDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    BOOL flag;

    switch (message) {
        case WM_INITDIALOG:
            // Set initial values of the parameters
            SetDlgItemInt(hDlg,IDC_X,arg[CurWnd].x,0);
            SetDlgItemInt(hDlg,IDC_Y,arg[CurWnd].y,0);
            SetDlgItemInt(hDlg,IDC_WIDTH,arg[CurWnd].width,0);
            SetDlgItemInt(hDlg,IDC_HEIGHT,arg[CurWnd].height,0);
            SendMessage(GetDlgItem(hDlg,IDC_FITTED_VIEW),BM_SETCHECK, arg[CurWnd].FittedView, 0L);
            if (arg[CurWnd].InputType=='F') flag=FALSE;else flag=TRUE;
            SendMessage(GetDlgItem(hDlg,IDC_FILE_OPT),BM_SETCHECK, flag, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_VER_BAR),BM_SETCHECK, arg[CurWnd].ShowVerBar, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_HOR_BAR),BM_SETCHECK, arg[CurWnd].ShowHorBar, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_BORDER),BM_SETCHECK, arg[CurWnd].BorderMargins, 0L);
            SendMessage(GetDlgItem(hDlg,IDC_CHILD),BM_SETCHECK, arg[CurWnd].style&WS_CHILD?TRUE:FALSE, 0L);

            lstrcpy(PrevFile,arg[CurWnd].file);  // store the previous file name
            SetDlgItemText(hDlg,IDC_FILE_NAME,arg[CurWnd].file);

            return (TRUE);

        case WM_COMMAND:
            switch (wParam) {
                case IDC_EDIT:
                case IDC_VIEW:
                   arg[CurWnd].ReadOnly=(wParam==IDC_VIEW)?TRUE:FALSE;

                   if (!AssignIntValue(hDlg,IDC_X,&arg[CurWnd].x)) return (TRUE);
                   if (!AssignIntValue(hDlg,IDC_Y,&arg[CurWnd].y)) return (TRUE);
                   if (!AssignIntValue(hDlg,IDC_WIDTH,&arg[CurWnd].width)) return (TRUE);
                   if (!AssignIntValue(hDlg,IDC_HEIGHT,&arg[CurWnd].height)) return (TRUE);
                   arg[CurWnd].FittedView=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_FITTED_VIEW),BM_GETCHECK, 0, 0L);
                   flag=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_FILE_OPT),BM_GETCHECK, 0, 0L);
                   if (flag) arg[CurWnd].InputType='B';else arg[CurWnd].InputType='F';
                   arg[CurWnd].ShowVerBar=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_VER_BAR),BM_GETCHECK, 0, 0L);
                   arg[CurWnd].ShowHorBar=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_HOR_BAR),BM_GETCHECK, 0, 0L);
                   arg[CurWnd].BorderMargins=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_BORDER),BM_GETCHECK, 0, 0L);

                   flag=(BOOL)SendMessage(GetDlgItem(hDlg,IDC_CHILD),BM_GETCHECK, 0, 0L);
                   if (flag) {
                      arg[CurWnd].style=WS_CHILD|WS_THICKFRAME|WS_BORDER|WS_MINIMIZEBOX|WS_MAXIMIZEBOX;
                      #if !defined(WIN32)
                         arg[CurWnd].style|=WS_CAPTION;  // can't add 'caption' style for win32 'cause the 'edit' control never gains focus if the parent is also a child and has the 'caption' on.
                      #endif
                   }
                   else      arg[CurWnd].style=WS_OVERLAPPEDWINDOW;

                   GetDlgItemText(hDlg,IDC_FILE_NAME,arg[CurWnd].file,100);

                   EndDialog(hDlg, TRUE);
                   return (TRUE);

                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}


/****************************************************************************
    DemoSelectWin:
    This dialog box allows you to select a TER window.
****************************************************************************/
int CALLBACK _export DemoSelectWin(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int i,j;
    char string[64];

    switch (message) {
        case WM_INITDIALOG:
            // insert the open file names in the dialog box
            for (i=0;i<MAX_WINDS;i++) {
               if (arg[i].open) {
                  if (arg[i].file[0]==0) SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD_PTR)(LPSTR)"UNNAMED");
                  else                   SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_ADDSTRING, 0, (DWORD_PTR)(LPSTR)arg[i].file);
               }
            }
            SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_SETCURSEL, 0,0L);
            SetFocus(GetDlgItem(hDlg,IDC_BOX));
            return (FALSE);

        case WM_COMMAND:
            switch (wParam) {
                case IDC_BOX:
                    if (HIWORD(lParam)!=LBN_DBLCLK) break;

                case IDOK:
                   j=(int)SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_GETCURSEL, 0,0L);
                   SendMessage(GetDlgItem(hDlg,IDC_BOX),LB_GETTEXT, j,(DWORD_PTR)(LPSTR)string);
                   if (lstrcmp(string,"UNNAMED")==0) string[0]=0;
                   for (i=0;i<MAX_WINDS;i++) {
                      if (arg[i].open && lstrcmp(arg[i].file,string)==0) {
                        EndDialog(hDlg, i);
                        return (TRUE);
                      }
                   }
                   EndDialog(hDlg, -1);
                   return (TRUE);

                case IDCANCEL:
                   EndDialog(hDlg, -1);
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

